package com.sun.media.sound;

import java.io.IOException;
import javax.sound.sampled.AudioInputStream;
import java.util.Vector;
import javax.sound.sampled.AudioFormat;

public final class PCMtoPCMCodec extends SunCodec
{
    private static final AudioFormat.Encoding[] inputEncodings;
    private static final AudioFormat.Encoding[] outputEncodings;
    private static final int tempBufferSize = 64;
    private byte[] tempBuffer;
    
    public PCMtoPCMCodec() {
        super(PCMtoPCMCodec.inputEncodings, PCMtoPCMCodec.outputEncodings);
        this.tempBuffer = null;
    }
    
    @Override
    public AudioFormat.Encoding[] getTargetEncodings(AudioFormat audioFormat) {
        if (audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_SIGNED) || audioFormat.getEncoding().equals(AudioFormat.Encoding.PCM_UNSIGNED)) {
            return new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED };
        }
        return new AudioFormat.Encoding[0];
    }
    
    @Override
    public AudioFormat[] getTargetFormats(AudioFormat.Encoding obj, AudioFormat audioFormat) {
        AudioFormat[] outputFormats = this.getOutputFormats(audioFormat);
        Vector<Object> vector = new Vector();
        for (int i = 0; i < outputFormats.length; ++i) {
            if (outputFormats[i].getEncoding().equals(obj)) {
                vector.addElement(outputFormats[i]);
            }
        }
        AudioFormat[] array = new AudioFormat[vector.size()];
        for (int j = 0; j < array.length; ++j) {
            array[j] = (AudioFormat)(AudioFormat)vector.elementAt(j);
        }
        return array;
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat.Encoding encoding, AudioInputStream audioInputStream) {
        if (this.isConversionSupported(encoding, audioInputStream.getFormat())) {
            AudioFormat format = audioInputStream.getFormat();
            return this.getAudioInputStream(new AudioFormat(encoding, format.getSampleRate(), format.getSampleSizeInBits(), format.getChannels(), format.getFrameSize(), format.getFrameRate(), format.isBigEndian()), audioInputStream);
        }
        throw new IllegalArgumentException("Unsupported conversion: " + audioInputStream.getFormat().toString() + " to " + encoding.toString());
    }
    
    @Override
    public AudioInputStream getAudioInputStream(AudioFormat audioFormat, AudioInputStream audioInputStream) {
        return this.getConvertedStream(audioFormat, audioInputStream);
    }
    
    private AudioInputStream getConvertedStream(AudioFormat format, AudioInputStream audioInputStream) {
        if (audioInputStream.getFormat().matches(format)) {
            return audioInputStream;
        }
        AudioInputStream audioInputStream2 = new PCMtoPCMCodec.PCMtoPCMCodecStream(audioInputStream, format);
        this.tempBuffer = new byte[64];
        return audioInputStream2;
    }
    
    private AudioFormat[] getOutputFormats(AudioFormat audioFormat) {
        Vector<AudioFormat> vector = new Vector();
        int sampleSizeInBits = audioFormat.getSampleSizeInBits();
        int bigEndian = audioFormat.isBigEndian();
        if (sampleSizeInBits == 8) {
            if (AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding())) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
            }
            if (AudioFormat.Encoding.PCM_UNSIGNED.equals(audioFormat.getEncoding())) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
            }
        }
        else if (sampleSizeInBits == 16) {
            if (AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding()) && bigEndian) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
            }
            if (AudioFormat.Encoding.PCM_UNSIGNED.equals(audioFormat.getEncoding()) && bigEndian) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
            }
            if (AudioFormat.Encoding.PCM_SIGNED.equals(audioFormat.getEncoding()) && bigEndian == 0) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
            }
            if (AudioFormat.Encoding.PCM_UNSIGNED.equals(audioFormat.getEncoding()) && bigEndian == 0) {
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), false));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_UNSIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
                vector.addElement(new AudioFormat(AudioFormat.Encoding.PCM_SIGNED, audioFormat.getSampleRate(), audioFormat.getSampleSizeInBits(), audioFormat.getChannels(), audioFormat.getFrameSize(), audioFormat.getFrameRate(), true));
            }
        }
        AudioFormat[] array;
        synchronized (vector) {
            array = new AudioFormat[vector.size()];
            for (int i = 0; i < array.length; ++i) {
                array[i] = (AudioFormat)(AudioFormat)vector.elementAt(i);
            }
        }
        return array;
    }
    
    static {
        inputEncodings = new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED };
        outputEncodings = new AudioFormat.Encoding[] { AudioFormat.Encoding.PCM_SIGNED, AudioFormat.Encoding.PCM_UNSIGNED };
    }
    
    class PCMtoPCMCodecStream extends AudioInputStream
    {
        private final int PCM_SWITCH_SIGNED_8BIT = 1;
        private final int PCM_SWITCH_ENDIAN = 2;
        private final int PCM_SWITCH_SIGNED_LE = 3;
        private final int PCM_SWITCH_SIGNED_BE = 4;
        private final int PCM_UNSIGNED_LE2SIGNED_BE = 5;
        private final int PCM_SIGNED_LE2UNSIGNED_BE = 6;
        private final int PCM_UNSIGNED_BE2SIGNED_LE = 7;
        private final int PCM_SIGNED_BE2UNSIGNED_LE = 8;
        private final int sampleSizeInBytes;
        private int conversionType;
        
        PCMtoPCMCodecStream(AudioInputStream stream, AudioFormat audioFormat) {
            super(stream, audioFormat, -1L);
            this.conversionType = 0;
            AudioFormat format = stream.getFormat();
            if (!PCMtoPCMCodec.this.isConversionSupported(format, audioFormat)) {
                throw new IllegalArgumentException("Unsupported conversion: " + format.toString() + " to " + audioFormat.toString());
            }
            AudioFormat.Encoding encoding = format.getEncoding();
            AudioFormat.Encoding encoding2 = audioFormat.getEncoding();
            int bigEndian = format.isBigEndian();
            int bigEndian2 = audioFormat.isBigEndian();
            int sampleSizeInBits = format.getSampleSizeInBits();
            this.sampleSizeInBytes = sampleSizeInBits / 8;
            if (sampleSizeInBits == 8) {
                if (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) && AudioFormat.Encoding.PCM_SIGNED.equals(encoding2)) {
                    this.conversionType = 1;
                }
                else if (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding2)) {
                    this.conversionType = 1;
                }
            }
            else if (encoding.equals(encoding2) && bigEndian != bigEndian2) {
                this.conversionType = 2;
            }
            else if (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) && bigEndian == 0 && AudioFormat.Encoding.PCM_SIGNED.equals(encoding2) && bigEndian2) {
                this.conversionType = 5;
            }
            else if (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && bigEndian == 0 && AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding2) && bigEndian2) {
                this.conversionType = 6;
            }
            else if (AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding) && bigEndian && AudioFormat.Encoding.PCM_SIGNED.equals(encoding2) && bigEndian2 == 0) {
                this.conversionType = 7;
            }
            else if (AudioFormat.Encoding.PCM_SIGNED.equals(encoding) && bigEndian && AudioFormat.Encoding.PCM_UNSIGNED.equals(encoding2) && bigEndian2 == 0) {
                this.conversionType = 8;
            }
            this.frameSize = format.getFrameSize();
            if (this.frameSize == -1) {
                this.frameSize = 1;
            }
            if ((stream instanceof AudioInputStream)) {
                this.frameLength = stream.getFrameLength();
            }
            else {
                this.frameLength = -1L;
            }
            this.framePos = 0L;
            return;
        }
        
        @Override
        public int read() throws IOException {
            if (this.frameSize == 1) {
                if (this.conversionType == 1) {
                    int read = super.read();
                    if (read < 0) {
                        return read;
                    }
                    int n = (byte)(read & 0xF);
                    return ((n >= 0) ? ((byte)(0x80 | n)) : ((byte)(0x7F & n))) & 0xF;
                }
                else {
                    throw new IOException("cannot read a single byte if frame size > 1");
                }
            }
            else {
                throw new IOException("cannot read a single byte if frame size > 1");
            }
        }
        
        @Override
        public int read(byte[] array) throws IOException {
            return this.read(array, 0, array.length);
        }
        
        @Override
        public int read(byte[] b, int off, int len) throws IOException {
            if (len % this.frameSize != 0) {
                len -= len % this.frameSize;
            }
            if (this.frameLength != -1L && (long)(len / this.frameSize) > this.frameLength - this.framePos) {
                len = (int)(this.frameLength - this.framePos) * this.frameSize;
            }
            int read = super.read(b, off, len);
            if (read < 0) {
                return read;
            }
            switch (this.conversionType) {
                case 1: {
                    this.switchSigned8bit(b, off, len, read);
                    break;
                }
                case 2: {
                    this.switchEndian(b, off, len, read);
                    break;
                }
                case 3: {
                    this.switchSignedLE(b, off, len, read);
                    break;
                }
                case 4: {
                    this.switchSignedBE(b, off, len, read);
                    break;
                }
                case 5:
                case 6: {
                    this.switchSignedLE(b, off, len, read);
                    this.switchEndian(b, off, len, read);
                    break;
                }
                case 7:
                case 8: {
                    this.switchSignedBE(b, off, len, read);
                    this.switchEndian(b, off, len, read);
                    break;
                }
            }
            return read;
        }
        
        private void switchSigned8bit(byte[] array, int i, int n, int n2) {
            for (; i < i + n2; ++i) {
                array[i] = ((array[i] >= 0) ? ((byte)(0x80 | array[i])) : ((byte)(0x7F & array[i])));
            }
        }
        
        private void switchSignedBE(byte[] array, int i, int n, int n2) {
            for (; i < i + n2; i += this.sampleSizeInBytes) {
                array[i] = ((array[i] >= 0) ? ((byte)(0x80 | array[i])) : ((byte)(0x7F & array[i])));
            }
        }
        
        private void switchSignedLE(byte[] array, int n, int n2, int n3) {
            for (int i = n + this.sampleSizeInBytes - 1; i < n + n3; i += this.sampleSizeInBytes) {
                array[i] = ((array[i] >= 0) ? ((byte)(0x80 | array[i])) : ((byte)(0x7F & array[i])));
            }
        }
        
        private void switchEndian(byte[] array, int i, int n, int n2) {
            if (this.sampleSizeInBytes == 2) {
                int n3;
                for (; i < i + n2; i += this.sampleSizeInBytes) {
                    n3 = array[i];
                    array[i] = array[i + 1];
                    array[i + 1] = n3;
                }
            }
        }
    }
}
